@@ -188,7 +188,7 @@ ROLE(GROUPING,
 
 ROLE(SEPARATOR,
      "separator",
-     nsGkAtoms::separator_,
+     nsGkAtoms::separator,
      ATK_ROLE_SEPARATOR,
      NSAccessibilitySplitterRole,
      @"AXContentSeparator",
@@ -305,7 +305,7 @@ ROLE(LINK,
 
 ROLE(LIST,
      "list",
-     nsGkAtoms::list_,
+     nsGkAtoms::list,
      ATK_ROLE_LIST,
      NSAccessibilityListRole,
      NSAccessibilityContentListSubrole,
@@ -660,7 +660,7 @@ ROLE(PASSWORD_TEXT,
      nullptr,
      ATK_ROLE_PASSWORD_TEXT,
      NSAccessibilityTextFieldRole,
-     NSAccessibilityUnknownSubrole,
+     NSAccessibilitySecureTextFieldSubrole,
      ROLE_SYSTEM_TEXT,
      ROLE_SYSTEM_TEXT,
      java::SessionAccessibility::CLASSNAME_EDITTEXT,
@@ -943,7 +943,7 @@ ROLE(GRID_CELL,
 
 ROLE(NOTE,
      "note",
-     nsGkAtoms::note_,
+     nsGkAtoms::note,
      ATK_ROLE_COMMENT,
      NSAccessibilityGroupRole,
      @"AXDocumentNote",
@@ -985,7 +985,7 @@ ROLE(DEFINITION_LIST,
      nullptr,
      ATK_ROLE_LIST,
      NSAccessibilityListRole,
-     @"AXDescriptionList",
+     @"AXDefinitionList",
      ROLE_SYSTEM_LIST,
      ROLE_SYSTEM_LIST,
      java::SessionAccessibility::CLASSNAME_LISTVIEW,
@@ -1011,7 +1011,7 @@ ROLE(DEFINITION,
      nsGkAtoms::definition,
      ATK_ROLE_PARAGRAPH,
      NSAccessibilityGroupRole,
-     @"AXDescription",
+     @"AXDefinition",
      ROLE_SYSTEM_GROUPING,
      IA2_ROLE_PARAGRAPH,
      java::SessionAccessibility::CLASSNAME_VIEW,
@@ -1844,4 +1844,17 @@ ROLE(ROWGROUP,
      IsAccessibilityElementRule::IfChildlessWithNameAndFocusable,
      UIA_GroupControlTypeId,
      eNameFromSubtreeIfReqRule)
+
+ROLE(SEARCHBOX,
+     "searchbox",
+     nsGkAtoms::searchbox,
+     ATK_ROLE_ENTRY,
+     NSAccessibilityTextFieldRole,
+     @"AXSearchField",
+     ROLE_SYSTEM_TEXT,
+     ROLE_SYSTEM_TEXT,
+     java::SessionAccessibility::CLASSNAME_EDITTEXT,
+     IsAccessibilityElementRule::Yes,
+     UIA_EditControlTypeId,
+     eNameFromValueRule)
 // clang-format on
