@@ -14,7 +14,7 @@ import {
 } from "modules/ASRouterPreferences.sys.mjs";
 import { ASRouterTriggerListeners } from "modules/ASRouterTriggerListeners.sys.mjs";
 import { CFRPageActions } from "modules/CFRPageActions.sys.mjs";
-import { GlobalOverrider } from "test/unit/utils";
+import { GlobalOverrider } from "tests/unit/utils";
 import { PanelTestProvider } from "modules/PanelTestProvider.sys.mjs";
 import ProviderResponseSchema from "content-src/schemas/provider-response.schema.json";
 
@@ -170,6 +170,9 @@ describe("ASRouter", () => {
         scoreThreshold: 5000,
         isChinaRepack: false,
         userId: "adsf",
+        currentProfileId: "1",
+        canCreateSelectableProfiles: false,
+        hasSelectableProfiles: false,
       },
     };
     gBrowser = {
@@ -229,15 +232,21 @@ describe("ASRouter", () => {
       "spotlight",
       "moments-page",
       "pbNewtab",
+      "fxms-message-15",
     ].reduce((features, featureId) => {
       features[featureId] = {
-        getAllVariables: sandbox.stub().returns(null),
+        getEnrollmentMetadata: sandbox.stub().returns({
+          slug: "experiment-slug",
+          branch: "experiment-branch-slug",
+          isRollout: false,
+        }),
+        getAllVariables: sandbox.stub().returns(undefined),
         recordExposureEvent: sandbox.stub(),
       };
       return features;
     }, {});
     globals.set({
-      // Testing framework doesn't know how to `defineLazyModuleGetters` so we're
+      // Testing framework doesn't know how to `defineESModuleGetters` so we're
       // importing these modules into the global scope ourselves.
       GroupsConfigurationProvider: { getMessages: () => Promise.resolve([]) },
       ASRouterPreferences,
@@ -261,30 +270,16 @@ describe("ASRouter", () => {
           return this;
         }
         getRecord() {
-          return Promise.resolve({ data: {} });
+          return Promise.resolve({ data: { attachment: { size: 42 } } });
         }
       },
-      Downloader: class {
+      UnstoredDownloader: class {
         download() {
-          return Promise.resolve("/path/to/download");
+          return Promise.resolve({ buffer: "fake buffer" });
         }
       },
       NimbusFeatures: fakeNimbusFeatures,
       ExperimentAPI: {
-        getExperimentMetaData: sandbox.stub().returns({
-          slug: "experiment-slug",
-          active: true,
-          branch: { slug: "experiment-branch-slug" },
-        }),
-        getExperiment: sandbox.stub().returns({
-          branch: {
-            slug: "unit-slug",
-            feature: {
-              featureId: "foo",
-              value: { id: "test-message" },
-            },
-          },
-        }),
         getAllBranches: sandbox.stub().resolves([]),
         ready: sandbox.stub().resolves(),
       },
@@ -304,6 +299,9 @@ describe("ASRouter", () => {
         // This is just a subset of supported locales that happen to be used in
         // the test.
         isLocaleSupported: locale => ["en-US", "ja-JP-mac"].includes(locale),
+        // PathUtils.join() is mocked in `unit-entry.js`, only filenames count.
+        cfrFluentFileDir: "ms-language-packs",
+        cfrFluentFilePath: "asrouter.ftl",
       },
     });
     await createRouterAndInit();
@@ -497,9 +495,9 @@ describe("ASRouter", () => {
       );
     });
     describe("lazily loading local test providers", () => {
-      afterEach(() => {
-        Router.uninit();
-      });
+      let justIdAndContent = ({ id, content }) => ({ id, content });
+      afterEach(() => Router.uninit());
+
       it("should add the local test providers on init if devtools are enabled", async () => {
         sandbox.stub(ASRouterPreferences, "devtoolsEnabled").get(() => true);
 
@@ -514,6 +512,38 @@ describe("ASRouter", () => {
 
         assert.notProperty(Router._localProviders, "PanelTestProvider");
       });
+      it("should flatten experiment translated messages from local test providers if devtools are enabled...", async () => {
+        sandbox.stub(ASRouterPreferences, "devtoolsEnabled").get(() => true);
+
+        await createRouterAndInit();
+
+        assert.property(Router._localProviders, "PanelTestProvider");
+
+        expect(
+          Router.state.messages.map(justIdAndContent)
+        ).to.deep.include.members([
+          { id: "experimentL10n", content: { text: "UniqueText" } },
+        ]);
+      });
+      it("...but not if devtools are disabled", async () => {
+        sandbox.stub(ASRouterPreferences, "devtoolsEnabled").get(() => false);
+
+        await createRouterAndInit();
+
+        assert.notProperty(Router._localProviders, "PanelTestProvider");
+
+        let justIdAndContentMessages =
+          Router.state.messages.map(justIdAndContent);
+        expect(justIdAndContentMessages).not.to.deep.include.members([
+          { id: "experimentL10n", content: { text: "UniqueText" } },
+        ]);
+        expect(justIdAndContentMessages).to.deep.include.members([
+          {
+            id: "experimentL10n",
+            content: { text: { $l10n: { text: "UniqueText" } } },
+          },
+        ]);
+      });
     });
   });
 
@@ -530,7 +560,7 @@ describe("ASRouter", () => {
         Router.onPrefChange
       );
     });
-    it("should send a AS_ROUTER_TARGETING_UPDATE message", async () => {
+    it("should call clearChildMessages (does nothing, see bug 1899028)", async () => {
       const messageTargeted = {
         id: "1",
         campaign: "foocampaign",
@@ -840,12 +870,9 @@ describe("ASRouter", () => {
       assertRouterContainsMessages(FAKE_LOCAL_MESSAGES);
     });
     it("should parse the triggers in the messages and register the trigger listeners", async () => {
-      sandbox.spy(
-        ASRouterTriggerListeners.get("openURL"),
-        "init"
-      ); /* eslint-disable object-property-newline */
+      sandbox.spy(ASRouterTriggerListeners.get("openURL"), "init");
 
-      /* eslint-disable object-curly-newline */ await createRouterAndInit([
+      await createRouterAndInit([
         {
           id: "foo",
           type: "local",
@@ -874,10 +901,8 @@ describe("ASRouter", () => {
             },
           ],
         },
-      ]); /* eslint-enable object-property-newline */
-      /* eslint-enable object-curly-newline */ assert.calledTwice(
-        ASRouterTriggerListeners.get("openURL").init
-      );
+      ]);
+      assert.calledTwice(ASRouterTriggerListeners.get("openURL").init);
       assert.calledWithExactly(
         ASRouterTriggerListeners.get("openURL").init,
         Router._triggerHandler,
@@ -899,10 +924,10 @@ describe("ASRouter", () => {
           enabled: true,
           messages: [
             {
-              id: "bar3",
+              id: "foo",
               template: "simple_template",
               trigger: { id: "messagesLoaded" },
-              content: { title: "Bar3", body: "Bar123" },
+              content: { title: "Foo", body: "Bar123" },
             },
           ],
         },
@@ -917,6 +942,26 @@ describe("ASRouter", () => {
         true
       );
     });
+    it("should not register a trigger listener in automation for a message with skip_in_tests", async () => {
+      sandbox.spy(ASRouterTriggerListeners.get("openURL"), "init");
+      await createRouterAndInit([
+        {
+          id: "foo",
+          type: "local",
+          enabled: true,
+          messages: [
+            {
+              id: "foo",
+              template: "simple_template",
+              trigger: { id: "openURL" },
+              content: { title: "Foo", body: "Foo123" },
+              skip_in_tests: "testing",
+            },
+          ],
+        },
+      ]);
+      assert.notCalled(ASRouterTriggerListeners.get("openURL").init);
+    });
     it("should gracefully handle messages loading before a window or browser exists", async () => {
       sandbox.stub(global, "gBrowser").value(undefined);
       sandbox
@@ -959,14 +1004,13 @@ describe("ASRouter", () => {
         .rejects("fake error");
       await createRouterAndInit();
       assert.calledWith(initParams.dispatchCFRAction, {
+        type: "AS_ROUTER_TELEMETRY_USER_EVENT",
         data: {
           action: "asrouter_undesired_event",
+          message_id: "n/a",
           event: "ASR_RS_ERROR",
           event_context: "remotey-settingsy",
-          message_id: "n/a",
         },
-        meta: { from: "ActivityStream:Content", to: "ActivityStream:Main" },
-        type: "AS_ROUTER_TELEMETRY_USER_EVENT",
       });
     });
     it("should dispatch undesired event if RemoteSettings returns no messages", async () => {
@@ -974,14 +1018,13 @@ describe("ASRouter", () => {
         .stub(MessageLoaderUtils, "_getRemoteSettingsMessages")
         .resolves([]);
       assert.calledWith(initParams.dispatchCFRAction, {
+        type: "AS_ROUTER_TELEMETRY_USER_EVENT",
         data: {
           action: "asrouter_undesired_event",
+          message_id: "n/a",
           event: "ASR_RS_NO_MESSAGES",
           event_context: "remotey-settingsy",
-          message_id: "n/a",
         },
-        meta: { from: "ActivityStream:Content", to: "ActivityStream:Main" },
-        type: "AS_ROUTER_TELEMETRY_USER_EVENT",
       });
     });
     it("should download the attachment if RemoteSettings returns some messages", async () => {
@@ -991,8 +1034,9 @@ describe("ASRouter", () => {
       sandbox
         .stub(MessageLoaderUtils, "_getRemoteSettingsMessages")
         .resolves([{ id: "message_1" }]);
+      sandbox.stub(global.IOUtils, "exists").resolves(false);
       const spy = sandbox.spy();
-      global.Downloader.prototype.downloadToDisk = spy;
+      global.UnstoredDownloader.prototype.download = spy;
       const provider = {
         id: "cfr",
         enabled: true,
@@ -1022,14 +1066,13 @@ describe("ASRouter", () => {
       await createRouterAndInit([provider]);
 
       assert.calledWith(initParams.dispatchCFRAction, {
+        type: "AS_ROUTER_TELEMETRY_USER_EVENT",
         data: {
           action: "asrouter_undesired_event",
+          message_id: "n/a",
           event: "ASR_RS_NO_MESSAGES",
           event_context: "ms-language-packs",
-          message_id: "n/a",
         },
-        meta: { from: "ActivityStream:Content", to: "ActivityStream:Main" },
-        type: "AS_ROUTER_TELEMETRY_USER_EVENT",
       });
     });
   });
@@ -1094,6 +1137,38 @@ describe("ASRouter", () => {
         providers: [{ id: "cfr" }, { id: "badge" }],
       }));
     });
+    it("should return no messages if shouldShowMessagesToProfile returns false", async () => {
+      sandbox.stub(Router, "shouldShowMessagesToProfile").returns(false);
+      await Router.setState(() => ({
+        messages: [
+          { id: "foo", provider: "cfr", groups: ["cfr"] },
+          { id: "bar", provider: "cfr", groups: ["cfr"] },
+        ],
+      }));
+      const result = await Router.handleMessageRequest({
+        provider: "cfr",
+      });
+      assert.isNull(result);
+    });
+    it("should return messages if shouldShowMessagesToProfile returns true", async () => {
+      sandbox.stub(Router, "shouldShowMessagesToProfile").returns(true);
+      await Router.setState(() => ({
+        messages: [
+          { id: "foo", provider: "cfr", groups: ["cfr"] },
+          { id: "bar", provider: "cfr", groups: ["cfr"] },
+        ],
+      }));
+      const result = await Router.handleMessageRequest({
+        provider: "cfr",
+      });
+      assert.isNotNull(result);
+      assert.calledWithMatch(ASRouterTargeting.findMatchingMessage, {
+        messages: [
+          { id: "foo", provider: "cfr", groups: ["cfr"] },
+          { id: "bar", provider: "cfr", groups: ["cfr"] },
+        ],
+      });
+    });
     it("should not return a blocked message", async () => {
       // Block all messages except the first
       await Router.setState(() => ({
@@ -1334,6 +1409,15 @@ describe("ASRouter", () => {
       assert.lengthOf(result, 1);
       assert.deepEqual(result[0], message1);
     });
+    it("should filter out messages with skip_in_tests when in automation", async () => {
+      await Router.setState(() => ({
+        messages: [
+          { id: "foo", provider: "cfr", skip_in_tests: "testing", groups: [] },
+        ],
+      }));
+      const result = await Router.handleMessageRequest({ provider: "cfr" });
+      assert.isNull(result);
+    });
   });
 
   describe("#uninit", () => {
@@ -1450,12 +1534,12 @@ describe("ASRouter", () => {
       assert.isEmpty(Router.state.messages.filter(Router.isUnblockedMessage));
     });
     it("should be able to add multiple items to the messageBlockList", async () => {
-      await await Router.blockMessageById(FAKE_BUNDLE.map(b => b.id));
+      await Router.blockMessageById(FAKE_BUNDLE.map(b => b.id));
       assert.isTrue(Router.state.messageBlockList.includes(FAKE_BUNDLE[0].id));
       assert.isTrue(Router.state.messageBlockList.includes(FAKE_BUNDLE[1].id));
     });
     it("should save the messageBlockList", async () => {
-      await await Router.blockMessageById(FAKE_BUNDLE.map(b => b.id));
+      await Router.blockMessageById(FAKE_BUNDLE.map(b => b.id));
       assert.calledWithExactly(Router._storage.set, "messageBlockList", [
         FAKE_BUNDLE[0].id,
         FAKE_BUNDLE[1].id,
@@ -1534,17 +1618,15 @@ describe("ASRouter", () => {
     let experimentAPIStub;
     let featureIds = ["cfr", "moments-page", "infobar", "spotlight"];
     beforeEach(() => {
-      let getExperimentMetaDataStub = sandbox.stub();
       let getAllBranchesStub = sandbox.stub();
       featureIds.forEach(feature => {
         global.NimbusFeatures[feature].getAllVariables.returns({
           id: `message-${feature}`,
         });
-        getExperimentMetaDataStub.withArgs({ featureId: feature }).returns({
+        global.NimbusFeatures[feature].getEnrollmentMetadata.returns({
           slug: `slug-${feature}`,
-          branch: {
-            slug: `branch-${feature}`,
-          },
+          branch: `branch-${feature}`,
+          isRollout: false,
         });
         getAllBranchesStub.withArgs(`slug-${feature}`).resolves([
           {
@@ -1554,7 +1636,6 @@ describe("ASRouter", () => {
         ]);
       });
       experimentAPIStub = {
-        getExperimentMetaData: getExperimentMetaDataStub,
         getAllBranches: getAllBranchesStub,
       };
       globals.set("ExperimentAPI", experimentAPIStub);
@@ -1599,7 +1680,6 @@ describe("ASRouter", () => {
       Router.loadMessagesFromAllProviders.onFirstCall().resolves();
 
       await Router.sendTriggerMessage({
-        tabId: 0,
         browser: gBrowser.selectedBrowser,
         id: "firstRun",
       });
@@ -1615,35 +1695,24 @@ describe("ASRouter", () => {
       );
     });
     it("should record telemetry information", async () => {
-      const startTelemetryStopwatch = sandbox.stub(
-        global.TelemetryStopwatch,
-        "start"
+      const fakeTimerId = 42;
+      const start = sandbox
+        .stub(global.Glean.messagingSystem.messageRequestTime, "start")
+        .returns(fakeTimerId);
+      const stopAndAccumulate = sandbox.stub(
+        global.Glean.messagingSystem.messageRequestTime,
+        "stopAndAccumulate"
       );
-      const finishTelemetryStopwatch = sandbox.stub(
-        global.TelemetryStopwatch,
-        "finish"
-      );
-
-      const tabId = 123;
 
       await Router.sendTriggerMessage({
-        tabId,
         browser: {},
         id: "firstRun",
       });
 
-      assert.calledTwice(startTelemetryStopwatch);
-      assert.calledWithExactly(
-        startTelemetryStopwatch,
-        "MS_MESSAGE_REQUEST_TIME_MS",
-        { tabId }
-      );
-      assert.calledTwice(finishTelemetryStopwatch);
-      assert.calledWithExactly(
-        finishTelemetryStopwatch,
-        "MS_MESSAGE_REQUEST_TIME_MS",
-        { tabId }
-      );
+      assert.calledTwice(start);
+      assert.calledWithExactly(start);
+      assert.calledTwice(stopAndAccumulate);
+      assert.calledWithExactly(stopAndAccumulate, fakeTimerId);
     });
     it("should have previousSessionEnd in the message context", () => {
       assert.propertyVal(
@@ -1681,15 +1750,14 @@ describe("ASRouter", () => {
         },
       ];
       sandbox.stub(Router, "handleMessageRequest").resolves(messages);
-      sandbox.spy(Services.telemetry, "recordEvent");
+      sandbox.spy(Glean.messagingExperiments.reachCfr, "record");
 
       await Router.sendTriggerMessage({
-        tabId: 0,
         browser: {},
         id: "foo",
       });
 
-      assert.calledTwice(Services.telemetry.recordEvent);
+      assert.calledTwice(Glean.messagingExperiments.reachCfr.record);
     });
     it("should not record the Reach event if it's already sent", async () => {
       let messages = [
@@ -1704,15 +1772,17 @@ describe("ASRouter", () => {
         },
       ];
       sandbox.stub(Router, "handleMessageRequest").resolves(messages);
-      sandbox.spy(Services.telemetry, "recordEvent");
+      sandbox.spy(Glean.messagingExperiments.reachCfr, "record");
 
       await Router.sendTriggerMessage({
-        tabId: 0,
         browser: {},
         id: "foo",
       });
-      assert.notCalled(Services.telemetry.recordEvent);
+      assert.notCalled(Glean.messagingExperiments.reachCfr.record);
     });
+    // XXX this next test set (ie the single `it` that tries to generate
+    // four tests with `forEach`) doesn't work, because it will always
+    // pass, so don't use it as a pattern to write other tests. Bug 1967593
     it("should record the Exposure event for each valid feature", async () => {
       ["cfr_doorhanger", "update_action", "infobar", "spotlight"].forEach(
         async template => {
@@ -1737,7 +1807,6 @@ describe("ASRouter", () => {
           sandbox.stub(Router, "handleMessageRequest").resolves(messages);
 
           await Router.sendTriggerMessage({
-            tabId: 0,
             browser: {},
             id: "foo",
           });
@@ -1748,6 +1817,52 @@ describe("ASRouter", () => {
         }
       );
     });
+
+    it("should send Exposure and route messages if recording reach fails", async () => {
+      const template = "feature_callout";
+      const featureId = "fxms-message-15";
+      const featureIdReachGroup = "FxmsMessage15";
+      let messages = [
+        {
+          _nimbusFeature: [featureId], // from _experimentsAPILoader
+          forReachEvent: {
+            sent: false,
+            group: featureIdReachGroup,
+          },
+          id: "foo1",
+          template,
+          trigger: { id: "fakeTrigger" },
+          content: { title: "Foo1", body: "Foo123-1" },
+        },
+        {
+          _nimbusFeature: [featureId], // from _experimentsAPILoader
+          id: "foo2",
+          template,
+          trigger: { id: "fakeTrigger" },
+          content: { title: "Foo2", body: "Foo123-2" },
+        },
+      ];
+      sandbox.stub(Router, "handleMessageRequest").resolves(messages);
+      sandbox.spy(Router, "routeCFRMessage");
+      sandbox
+        .stub(
+          Glean.messagingExperiments[`reach${featureIdReachGroup}`],
+          "record"
+        )
+        .throws(new Error("stuff"));
+      assert.notCalled(global.NimbusFeatures[featureId].recordExposureEvent);
+
+      await Router.sendTriggerMessage(
+        {
+          browser: {},
+          id: "foo",
+        },
+        true // skipMessagesLoaded to avoid irrelevant calls spy/stub calls
+      );
+
+      assert.calledOnce(global.NimbusFeatures[featureId].recordExposureEvent);
+      assert.calledOnce(Router.routeCFRMessage);
+    });
   });
 
   describe("forceAttribution", () => {
@@ -2432,11 +2547,8 @@ describe("ASRouter", () => {
 
       await MessageLoaderUtils.loadMessagesForProvider(args);
 
+      assert.calledOnce(global.NimbusFeatures.spotlight.getEnrollmentMetadata);
       assert.calledOnce(global.NimbusFeatures.spotlight.getAllVariables);
-      assert.calledOnce(global.ExperimentAPI.getExperimentMetaData);
-      assert.calledWithExactly(global.ExperimentAPI.getExperimentMetaData, {
-        featureId: "spotlight",
-      });
     });
     it("should handle the case of no experiments in the ExperimentAPI", async () => {
       const args = {
@@ -2444,8 +2556,6 @@ describe("ASRouter", () => {
         featureIds: ["infobar"],
       };
 
-      global.ExperimentAPI.getExperiment.returns(null);
-
       const result = await MessageLoaderUtils.loadMessagesForProvider(args);
 
       assert.lengthOf(result.messages, 0);
@@ -2456,6 +2566,7 @@ describe("ASRouter", () => {
         featureIds: ["infobar"],
       };
       const enrollment = {
+        slug: "enrollment01",
         branch: {
           slug: "branch01",
           infobar: {
@@ -2468,8 +2579,10 @@ describe("ASRouter", () => {
       global.NimbusFeatures.infobar.getAllVariables.returns(
         enrollment.branch.infobar.value
       );
-      global.ExperimentAPI.getExperimentMetaData.returns({
-        branch: { slug: enrollment.branch.slug },
+      global.NimbusFeatures.infobar.getEnrollmentMetadata.returns({
+        slug: enrollment.slug,
+        branch: enrollment.branch.slug,
+        isRollout: false,
       });
       global.ExperimentAPI.getAllBranches.returns([
         enrollment.branch,
@@ -2517,10 +2630,10 @@ describe("ASRouter", () => {
       global.NimbusFeatures.cfr.getAllVariables.returns(
         enrollment.branch.cfr.value
       );
-      global.ExperimentAPI.getExperimentMetaData.returns({
+      global.NimbusFeatures.cfr.getEnrollmentMetadata.returns({
         slug: enrollment.slug,
-        active: true,
-        branch: { slug: enrollment.branch.slug },
+        branch: enrollment.branch.slug,
+        isRollout: false,
       });
       global.ExperimentAPI.getAllBranches.resolves([
         enrollment.branch,
@@ -2569,15 +2682,15 @@ describe("ASRouter", () => {
         },
       };
 
-      // Nedds to match the `featureIds` value to return an enrollment
+      // Needs to match the `featureIds` value to return an enrollment
       // for that feature
       global.NimbusFeatures.cfr.getAllVariables.returns(
         enrollment.branch.cfr.value
       );
-      global.ExperimentAPI.getExperimentMetaData.returns({
+      global.NimbusFeatures.cfr.getEnrollmentMetadata.returns({
         slug: enrollment.slug,
-        active: true,
-        branch: { slug: enrollment.branch.slug },
+        branch: enrollment.branch.slug,
+        isRollout: false,
       });
       global.ExperimentAPI.getAllBranches.resolves([
         enrollment.branch,
@@ -2621,11 +2734,12 @@ describe("ASRouter", () => {
       sandbox
         .stub(MessageLoaderUtils, "_getRemoteSettingsMessages")
         .resolves([{ id: "message_1" }]);
-      spy = sandbox.spy();
-      global.Downloader.prototype.downloadToDisk = spy;
+      sandbox.stub(global.IOUtils, "exists").resolves(false);
+      spy = sandbox.spy(global.UnstoredDownloader.prototype.download);
+      global.UnstoredDownloader.prototype.download = spy;
     });
     it("should be called with the expected dir path", async () => {
-      const dlSpy = sandbox.spy(global, "Downloader");
+      const writeSpy = sandbox.spy(global.IOUtils, "write");
 
       sandbox
         .stub(global.Services.locale, "appLocaleAsBCP47")
@@ -2633,14 +2747,39 @@ describe("ASRouter", () => {
 
       await MessageLoaderUtils._remoteSettingsLoader(provider, {});
 
-      assert.calledWith(
-        dlSpy,
-        "main",
-        "ms-language-packs",
-        "browser",
-        "newtab"
+      assert.calledOnce(spy);
+      assert.calledWithMatch(
+        writeSpy,
+        "asrouter.ftl", // PathUtils.join() is mocked in `unit-entry.js` and only returns the filename.
+        sinon.match.any,
+        { tmpPath: "asrouter.ftl.tmp" }
       );
     });
+    it("should download if local file has different size", async () => {
+      global.IOUtils.exists.resolves(true);
+      sandbox.stub(global.IOUtils, "stat").resolves({ size: 1337 });
+      sandbox
+        .stub(global.Services.locale, "appLocaleAsBCP47")
+        .get(() => "en-US");
+
+      await MessageLoaderUtils._remoteSettingsLoader(provider, {});
+
+      assert.calledOnce(spy);
+    });
+    it("should not download if local file has same size", async () => {
+      global.IOUtils.exists.resolves(true);
+      sandbox.stub(global.IOUtils, "stat").resolves({ size: 42 });
+      sandbox
+        .stub(global.KintoHttpClient.prototype, "getRecord")
+        .resolves({ data: { attachment: { size: 42 } } });
+      sandbox
+        .stub(global.Services.locale, "appLocaleAsBCP47")
+        .get(() => "en-US");
+
+      await MessageLoaderUtils._remoteSettingsLoader(provider, {});
+
+      assert.notCalled(spy);
+    });
     it("should allow fetch for known locales", async () => {
       sandbox
         .stub(global.Services.locale, "appLocaleAsBCP47")
