@@ -792,9 +792,7 @@ class MenuItem {
     }
     let menuMap = gMenuMap.get(this.extension);
     if (!menuMap.has(parentId)) {
-      throw new ExtensionError(
-        `Could not find any MenuItem with id: ${parentId}`
-      );
+      throw new ExtensionError(`Cannot find menu item with id ${parentId}`);
     }
     for (let item = menuMap.get(parentId); item; item = item.parent) {
       if (item === this) {
@@ -1407,7 +1405,7 @@ this.menusInternal = class extends ExtensionAPIPersistent {
 
           let menuItem = gMenuMap.get(extension).get(id);
           if (!menuItem) {
-            return;
+            throw new ExtensionError(`Cannot find menu item with id ${id}`);
           }
 
           menuItem.setProps(updateProperties);
@@ -1421,11 +1419,13 @@ this.menusInternal = class extends ExtensionAPIPersistent {
           }
 
           let menuItem = gMenuMap.get(extension).get(id);
-          if (menuItem) {
-            const menuIds = [menuItem.id, ...menuItem.descendantIds];
-            menuItem.remove();
-            ExtensionMenus.deleteMenus(extension, menuIds);
+          if (!menuItem) {
+            throw new ExtensionError(`Cannot find menu item with id ${id}`);
           }
+
+          const menuIds = [menuItem.id, ...menuItem.descendantIds];
+          menuItem.remove();
+          ExtensionMenus.deleteMenus(extension, menuIds);
         },
 
         removeAll: async () => {
