@@ -5,10 +5,19 @@ const { CustomizableUITestUtils } = ChromeUtils.importESModule(
   "resource://testing-common/CustomizableUITestUtils.sys.mjs"
 );
 
+const { EnterprisePolicyTesting, PoliciesPrefTracker } =
+  ChromeUtils.importESModule(
+    "resource://testing-common/EnterprisePolicyTesting.sys.mjs"
+  );
+
 const { UrlClassifierTestUtils } = ChromeUtils.importESModule(
   "resource://testing-common/UrlClassifierTestUtils.sys.mjs"
 );
 
+const { ReportBrokenSite } = ChromeUtils.importESModule(
+  "resource:///modules/ReportBrokenSite.sys.mjs"
+);
+
 const BASE_URL =
   "https://example.com/browser/browser/components/reportbrokensite/test/browser/";
 
@@ -26,8 +35,6 @@ const PREFS = {
   REASON: "ui.new-webcompat-reporter.reason-dropdown",
   SEND_MORE_INFO: "ui.new-webcompat-reporter.send-more-info-link",
   NEW_REPORT_ENDPOINT: "ui.new-webcompat-reporter.new-report-endpoint",
-  REPORT_SITE_ISSUE_ENABLED: "extensions.webcompat-reporter.enabled",
-  PREFERS_CONTRAST_ENABLED: "layout.css.prefers-contrast.enabled",
   TOUCH_EVENTS: "dom.w3c_touch_events.enabled",
   USE_ACCESSIBILITY_THEME: "ui.useAccessibilityTheme",
 };
@@ -144,6 +151,35 @@ function isSelectedTab(win, tab) {
   is(selectedTab, tab);
 }
 
+async function setupPolicyEngineWithJson(json, customSchema) {
+  PoliciesPrefTracker.restoreDefaultValues();
+  if (typeof json != "object") {
+    let filePath = getTestFilePath(json ? json : "non-existing-file.json");
+    return EnterprisePolicyTesting.setupPolicyEngineWithJson(
+      filePath,
+      customSchema
+    );
+  }
+  return EnterprisePolicyTesting.setupPolicyEngineWithJson(json, customSchema);
+}
+
+async function ensureReportBrokenSiteDisabledByPolicy() {
+  await setupPolicyEngineWithJson({
+    policies: {
+      DisableFeedbackCommands: true,
+    },
+  });
+}
+
+registerCleanupFunction(async function resetPolicies() {
+  if (Services.policies.status != Ci.nsIEnterprisePolicies.INACTIVE) {
+    await setupPolicyEngineWithJson("");
+  }
+  EnterprisePolicyTesting.resetRunOnceState();
+  PoliciesPrefTracker.restoreDefaultValues();
+  PoliciesPrefTracker.stop();
+});
+
 function ensureReportBrokenSitePreffedOn() {
   Services.prefs.setBoolPref(PREFS.DATAREPORTING_ENABLED, true);
   Services.prefs.setBoolPref(PREFS.REPORTER_ENABLED, true);
@@ -154,14 +190,6 @@ function ensureReportBrokenSitePreffedOff() {
   Services.prefs.setBoolPref(PREFS.REPORTER_ENABLED, false);
 }
 
-function ensureReportSiteIssuePreffedOn() {
-  Services.prefs.setBoolPref(PREFS.REPORT_SITE_ISSUE_ENABLED, true);
-}
-
-function ensureReportSiteIssuePreffedOff() {
-  Services.prefs.setBoolPref(PREFS.REPORT_SITE_ISSUE_ENABLED, false);
-}
-
 function ensureSendMoreInfoEnabled() {
   Services.prefs.setBoolPref(PREFS.SEND_MORE_INFO, true);
 }
@@ -199,6 +227,10 @@ function isMenuItemDisabled(menuItem, itemDesc) {
   ok(menuItem.disabled, `${itemDesc} menu item is disabled`);
 }
 
+function waitForWebcompatComTab(gBrowser) {
+  return BrowserTestUtils.waitForNewTab(gBrowser, NEW_REPORT_ENDPOINT_TEST_URL);
+}
+
 class ReportBrokenSiteHelper {
   sourceMenu = undefined;
   win = undefined;
@@ -228,14 +260,18 @@ class ReportBrokenSiteHelper {
     return this.openPanel?.hasAttribute("panelopen");
   }
 
-  async open(triggerMenuItem) {
+  async click(triggerMenuItem) {
     const window = triggerMenuItem.ownerGlobal;
+    await EventUtils.synthesizeMouseAtCenter(triggerMenuItem, {}, window);
+  }
+
+  async open(triggerMenuItem) {
     const shownPromise = BrowserTestUtils.waitForEvent(
       this.mainView,
       "ViewShown"
     );
     const focusPromise = BrowserTestUtils.waitForEvent(this.URLInput, "focus");
-    await EventUtils.synthesizeMouseAtCenter(triggerMenuItem, {}, window);
+    await this.click(triggerMenuItem);
     await shownPromise;
     await focusPromise;
   }
@@ -285,7 +321,7 @@ class ReportBrokenSiteHelper {
   }
 
   async clickSendMoreInfo() {
-    const newTabPromise = this.waitForSendMoreInfoTab();
+    const newTabPromise = waitForWebcompatComTab(this.win.gBrowser);
     EventUtils.synthesizeMouseAtCenter(this.sendMoreInfoLink, {}, this.win);
     const newTab = await newTabPromise;
     const receivedData = await SpecialPowers.spawn(
@@ -562,10 +598,6 @@ class MenuHelper {
     throw new Error("Should be defined in derived class");
   }
 
-  get reportSiteIssue() {
-    throw new Error("Should be defined in derived class");
-  }
-
   get popup() {
     throw new Error("Should be defined in derived class");
   }
@@ -590,16 +622,31 @@ class MenuHelper {
     return isMenuItemHidden(this.reportBrokenSite, this.menuDescription);
   }
 
-  isReportSiteIssueDisabled() {
-    return isMenuItemDisabled(this.reportSiteIssue, this.menuDescription);
+  async clickReportBrokenSiteAndAwaitWebCompatTabData() {
+    const newTabPromise = waitForWebcompatComTab(this.win.gBrowser);
+    await this.clickReportBrokenSite();
+    const newTab = await newTabPromise;
+    const receivedData = await SpecialPowers.spawn(
+      newTab.linkedBrowser,
+      [],
+      async function () {
+        await content.wrappedJSObject.messageArrived;
+        return content.wrappedJSObject.message;
+      }
+    );
+
+    this.win.gBrowser.removeCurrentTab();
+    return receivedData;
   }
 
-  isReportSiteIssueEnabled() {
-    return isMenuItemEnabled(this.reportSiteIssue, this.menuDescription);
-  }
-
-  isReportSiteIssueHidden() {
-    return isMenuItemHidden(this.reportSiteIssue, this.menuDescription);
+  async clickReportBrokenSite() {
+    if (!this.opened) {
+      await this.open();
+    }
+    isMenuItemEnabled(this.reportBrokenSite, this.menuDescription);
+    const rbs = new ReportBrokenSiteHelper(this);
+    await rbs.click(this.reportBrokenSite);
+    return rbs;
   }
 
   async openReportBrokenSite() {
@@ -632,10 +679,6 @@ class AppMenuHelper extends MenuHelper {
     return this.getViewNode("appMenu-report-broken-site-button");
   }
 
-  get reportSiteIssue() {
-    return undefined;
-  }
-
   get popup() {
     return this.win.document.getElementById("appMenu-popup");
   }
@@ -651,38 +694,6 @@ class AppMenuHelper extends MenuHelper {
   }
 }
 
-class AppMenuHelpSubmenuHelper extends MenuHelper {
-  menuDescription = "AppMenu help sub-menu";
-
-  get reportBrokenSite() {
-    return this.getViewNode("appMenu_help_reportBrokenSite");
-  }
-
-  get reportSiteIssue() {
-    return this.getViewNode("appMenu_help_reportSiteIssue");
-  }
-
-  get popup() {
-    return this.win.document.getElementById("appMenu-popup");
-  }
-
-  async open() {
-    await new CustomizableUITestUtils(this.win).openMainMenu();
-
-    const anchor = this.win.document.getElementById("PanelUI-menu-button");
-    this.win.PanelUI.showHelpView(anchor);
-
-    const appMenuHelpSubview = this.getViewNode("PanelUI-helpView");
-    await BrowserTestUtils.waitForEvent(appMenuHelpSubview, "ViewShown");
-  }
-
-  async close() {
-    if (this.opened) {
-      await new CustomizableUITestUtils(this.win).hideMainMenu();
-    }
-  }
-}
-
 class HelpMenuHelper extends MenuHelper {
   menuDescription = "Help Menu";
 
@@ -694,10 +705,6 @@ class HelpMenuHelper extends MenuHelper {
     return this.win.document.getElementById("help_reportBrokenSite");
   }
 
-  get reportSiteIssue() {
-    return this.win.document.getElementById("help_reportSiteIssue");
-  }
-
   get popup() {
     return this.getViewNode("PanelUI-helpView");
   }
@@ -722,6 +729,12 @@ class HelpMenuHelper extends MenuHelper {
     return new ReportBrokenSiteHelper(this);
   }
 
+  async clickReportBrokenSite() {
+    await this.open();
+    this.reportBrokenSite.click();
+    return new ReportBrokenSiteHelper(this);
+  }
+
   async open() {
     const { helpMenu } = this;
     const promise = BrowserTestUtils.waitForEvent(helpMenu, "popupshown");
@@ -757,10 +770,6 @@ class ProtectionsPanelHelper extends MenuHelper {
     return this.getViewNode("protections-popup-report-broken-site-button");
   }
 
-  get reportSiteIssue() {
-    return undefined;
-  }
-
   get popup() {
     this.win.gProtectionsHandler._initializePopup();
     return this.win.document.getElementById("protections-popup");
@@ -791,10 +800,6 @@ function AppMenu(win = window) {
   return new AppMenuHelper(win);
 }
 
-function AppMenuHelpSubmenu(win = window) {
-  return new AppMenuHelpSubmenuHelper(win);
-}
-
 function HelpMenu(win = window) {
   return new HelpMenuHelper(win);
 }
@@ -833,6 +838,18 @@ async function tabTo(match, win = window) {
   return undefined;
 }
 
+function filterFrameworkDetectorFails(ping, expected) {
+  // the framework detector's frame-script may fail to run in low memory or other
+  // weird corner-cases, so we ignore the results in that case if they don't match.
+  if (!areObjectsEqual(ping.frameworks, expected.frameworks)) {
+    const { fastclick, mobify, marfeel } = ping.frameworks;
+    if (!fastclick && !mobify && !marfeel) {
+      console.info("Ignoring failure to get framework data");
+      expected.frameworks = ping.frameworks;
+    }
+  }
+}
+
 async function setupStrictETP() {
   await UrlClassifierTestUtils.addTestTrackers();
   registerCleanupFunction(() => {
@@ -848,6 +865,7 @@ async function setupStrictETP() {
         "urlclassifier.trackingTable",
         "content-track-digest256,mochitest2-track-simple",
       ],
+      ["browser.contentblocking.category", "strict"],
     ],
   });
 }
